﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Windows.Editors;
using System.Collections;
using Infragistics.Controls.Editors;
using Infragistics.Windows;
using System.Windows.Controls.Primitives;
using Infragistics.Controls.Editors.Primitives;

namespace XamMultiComboEditor
{
    public class XamMultiComboEditor : ValueEditor
    {
        #region Constructor
        public XamMultiComboEditor()
        {
            DefaultStyleKey = typeof(XamMultiComboEditor);
        }

        static XamMultiComboEditor()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(XamMultiComboEditor),
                new FrameworkPropertyMetadata(typeof(XamMultiComboEditor)));
        }

        #endregion

        #region Public Properties

        #region ItemSource


        public IEnumerable ItemSource
        {
            get { return (IEnumerable)GetValue(ItemSourceProperty); }
            set { SetValue(ItemSourceProperty, value); }
        }

        public static readonly DependencyProperty ItemSourceProperty =
            DependencyProperty.Register(
                "ItemSource",
                typeof(IEnumerable),
                typeof(XamMultiComboEditor),
                new UIPropertyMetadata(null));


        #endregion

        #region DisplayMemberPath


        public string DisplayMemberPath
        {
            get { return (string)GetValue(DisplayMemberPathProperty); }
            set { SetValue(DisplayMemberPathProperty, value); }
        }

        public static readonly DependencyProperty DisplayMemberPathProperty =
            DependencyProperty.Register(
                "DisplayMemberPath",
                typeof(string),
                typeof(XamMultiComboEditor),
                new UIPropertyMetadata(null));


        #endregion

        #region SelectedValuePath


        public string SelectedValuePath
        {
            get { return (string)GetValue(SelectedValuePathProperty); }
            set { SetValue(SelectedValuePathProperty, value); }
        }

        public static readonly DependencyProperty SelectedValuePathProperty =
            DependencyProperty.Register(
                "SelectedValuePath",
                typeof(string),
                typeof(XamMultiComboEditor),
                new UIPropertyMetadata(null));


        #endregion

        #region SelectedItem


        public object SelectedItem
        {
            get { return (object)GetValue(SelectedItemProperty); }
            set { SetValue(SelectedItemProperty, value); }
        }

        public static readonly DependencyProperty SelectedItemProperty =
            DependencyProperty.Register(
                "SelectedItem",
                typeof(object),
                typeof(XamMultiComboEditor),
                new UIPropertyMetadata(
                    null,
                    (o, e) =>
                    {
                        XamMultiComboEditor editor = o as XamMultiComboEditor;
                        if (e.NewValue != null)
                        {
                            if (string.IsNullOrEmpty(editor.SelectedValuePath))
                                editor.Value = e.NewValue;
                            else
                                editor.Value = e.NewValue.GetType().GetProperty(editor.SelectedValuePath).GetValue(e.NewValue, null);
                        }

                    }));


        #endregion

        #region Columns

        public MultiComboColumnsCollection ComboColumns
        {
            get { return (MultiComboColumnsCollection)GetValue(ComboColumnsProperty); }
            set { SetValue(ComboColumnsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ComboColumns.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ComboColumnsProperty =
            DependencyProperty.Register(
            "ComboColumns", 
            typeof(MultiComboColumnsCollection), 
            typeof(XamMultiComboEditor), 
            new UIPropertyMetadata(null));

        #endregion

        #region ContentTemplate

        public DataTemplate ContentTemplate
        {
            get { return (DataTemplate)GetValue(ContentTemplateProperty); }
            set { SetValue(ContentTemplateProperty, value); }
        }

        public static readonly DependencyProperty ContentTemplateProperty =
                DependencyProperty.Register(
                "ContentTemplate",
                typeof(DataTemplate),
                typeof(XamMultiComboEditor),
                new UIPropertyMetadata(null));

        #endregion

        #region Drop Down Width


        public double DropDownWidth
        {
            get { return (double)GetValue(DropDownWidthProperty); }
            set { SetValue(DropDownWidthProperty, value); }
        }

        // Using a DependencyProperty as the backing store for DropDownWidth.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty DropDownWidthProperty =
            DependencyProperty.Register(
                "DropDownWidth",
                typeof(double),
                typeof(XamMultiComboEditor), 
                new UIPropertyMetadata(-1.0));


        #endregion

        #endregion

        #region ValueEdtior Members
        public override bool CanEditType(Type type)
        {
            return true;
        }

        public override bool CanRenderType(Type type)
        {
            return true;
        }
        #endregion

        #region Base Overrides

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            ContentControl nonEditContent = GetTemplateChild("nonEditContent") as ContentControl;
            if (nonEditContent != null)
            {
                Binding binding = new Binding
                {
                    Source = this,
                };


                if (string.IsNullOrEmpty(DisplayMemberPath))
                    binding.Path = new PropertyPath("SelectedItem");
                else
                    binding.Path = new PropertyPath("SelectedItem." + DisplayMemberPath);

                nonEditContent.SetBinding(ContentControl.ContentProperty, binding);
            }
        }

        protected override object OnCoerceValue(object val)
        {

            if (string.IsNullOrEmpty(SelectedValuePath) || ItemSource == null)
                SelectedItem = val;
            else
            {
                var querySource = ItemSource.Cast<object>();

                var item = querySource.Where(obj => obj.GetType().GetProperty(SelectedValuePath).GetValue(obj, null).Equals(val)).FirstOrDefault();

                if (item != null)
                {
                    SelectedItem = item;
                }
            }
            return base.OnCoerceValue(val);


        }

        protected override void OnEditModeStarted(Infragistics.Windows.Editors.Events.EditModeStartedEventArgs args)
        {
            base.OnEditModeStarted(args);

            XamMultiColumnComboEditor multicombo = Utilities.GetDescendantFromType(this, typeof(XamMultiColumnComboEditor), false) as XamMultiColumnComboEditor;
            if (multicombo != null)
            {
                multicombo.DropDownOpened += (ss, ee) =>
                {
                    Popup popup = Utilities.GetDescendantFromName(ss as DependencyObject, "Popup") as Popup;
                    if (popup != null)
                    {
                        Border border = Utilities.GetDescendantFromName(popup.Child, "RootPopupElement") as Border;
                        if (border != null)
                        {
                            if (DropDownWidth < 1)
                                return;

                            border.Width = DropDownWidth;
                        }

                    }
                };

                SpecializedTextBox textBox = Utilities.GetDescendantFromType(multicombo, typeof(SpecializedTextBox), false) as SpecializedTextBox;
                if (textBox != null)
                {
                    textBox.Dispatcher.BeginInvoke(new Action(() =>
                        {
                            textBox.Focus();
                            textBox.SelectAll();
                        }), System.Windows.Threading.DispatcherPriority.Background, null);
                }
            }        


            if (multicombo != null && ComboColumns != null)
            {
                multicombo.AutoGenerateColumns = false;
                foreach (ComboColumn column in ComboColumns)
                {
                    multicombo.Columns.Add(column);
                }
            }
        }


        protected override void OnEditModeEnding(Infragistics.Windows.Editors.Events.EditModeEndingEventArgs args)
        {
            base.OnEditModeEnding(args);
            XamMultiColumnComboEditor multicombo = Utilities.GetDescendantFromType(this, typeof(XamMultiColumnComboEditor), false) as XamMultiColumnComboEditor;
            if (multicombo != null)
            {
                multicombo.DropDownOpened -= (ss, ee) => { };
            }
        }
        #endregion
    }
}
